# -*- coding: utf-8 -*-
from .simpleMethods import *
import numpy as np
import copy 

class trendDisMethod(simpleMethods):
    def __init__(self,x,y):
        simpleMethods.__init__(self,x) 
        self.y = y
        self.cut_range = []
        
    def __stat(self,bad=1):
        '''
        统计数据集各个区间的坏样本比例
        :param bad:
        :return:
        '''
        n = 20 if len(self.x)>=10 else len(self.x)
        self.equalSize(n)
        self.value = np.array(self.y)
        #self.x_array = np.array(self.x)
        self.range_table = {}
        self.down = []
        self.bad_list = []
        self.count_list = []
        #print(self.range_dict)
        #print('for')
        for r in self.range_dict:
            range_value = self.value[(self.x<r[1]) & (self.x>=r[0])]
            bad_num = len(range_value[range_value==bad])
            count_num = len(range_value)
            if count_num == 0:
                rate = 0
            else:
                rate = bad_num/count_num
            
            self.range_table[r[1]] = {'bad_rate':rate,'count':count_num,'down':r[1],'range':r,
                                      'bad_num':bad_num,'count_num':count_num}
            self.down.append(r[1])
            self.bad_list.append(bad_num)
            self.count_list.append(count_num)
        print(self.range_table)

            
    def fit(self, bad=1, trend='up'):
        self.__stat(bad=bad)
        self.down = np.array(self.down)              # 各分区区间下限
        self.bad_list = np.array(self.bad_list)      # 各分区坏样本比例
        self.count_list = np.array(self.count_list)  # 各分区总样本比例
        
        self.cut_range = self.find_cut(trend=trend)  # 第一个切割点

        while True:
            cut_list = self.find_cut(self.cut_range,trend=trend)

            if len(cut_list)>0:
                for c in cut_list:
                    self.cut_range.append(c)
                    self.cut_range = sorted(list(set(self.cut_range)))
            else:
                break

        self.bins = np.array(sorted(list(set(self.cut_range))))
        
    
    def find_cut(self,cut_list=[],trend='up'):
        cuts = []
        if cut_list == []:
            candidate = sorted(copy.deepcopy(self.down))
            cut, rate = self.__find_cut(list(candidate),start='',end='',trend=trend)
            if rate:
                cuts.append(cut)
        else:
            for i in range(len(cut_list)):
                if i == 0:
                    candidate = list(self.down[self.down<cut_list[i]])
                    cut, rate = self.__find_cut(list(candidate),start='',end=cut_list[i],trend=trend)

                else:
                    candidate = list(self.down[(self.down<cut_list[i]) & (self.down>cut_list[i-1])])
                    cut, rate = self.__find_cut(list(candidate),start=cut_list[i-1],end=cut_list[i],trend=trend)

                if rate and cut not in self.cut_range:
                    cuts.append(cut)
                if i == len(cut_list)-1:
                    candidate = list(self.down[self.down>cut_list[i]])
                    cut, rate = self.__find_cut(list(candidate),start=cut_list[i],end='',trend=trend)

                    if rate and cut not in self.cut_range:
                        cuts.append(cut)   
        return cuts                       
        
    def __find_cut(self,candidate,start='',end='',trend='up'):
        result_cut = None
        result_rate = None

        for i in range(len(candidate)):
            if start=='' and end=='':
                bad_up = sum(self.bad_list[(self.down<=candidate[i])])
                count_up = sum(self.count_list[(self.down<=candidate[i])])                                                    
                bad_down = sum(self.bad_list[(self.down>candidate[i])])
                count_down = sum(self.count_list[(self.down>candidate[i])])    
            elif start == '' and end != '':   
                bad_up = sum(self.bad_list[(self.down<=candidate[i])])
                count_up = sum(self.count_list[(self.down<=candidate[i])])                                                    
                bad_down = sum(self.bad_list[(self.down>candidate[i]) & (self.down<=end)])
                count_down = sum(self.count_list[(self.down>candidate[i]) & (self.down<=end)])
            elif start != '' and end == '':
                bad_up = sum(self.bad_list[(self.down<=candidate[i]) & (self.down>start)])
                count_up = sum(self.count_list[(self.down<=candidate[i]) & (self.down>start)])                                                    
                bad_down = sum(self.bad_list[(self.down>candidate[i])])
                count_down = sum(self.count_list[(self.down>candidate[i])])    
            elif start != '' and end != '':
                bad_up = sum(self.bad_list[(self.down<=candidate[i]) & (self.down>start)])
                count_up = sum(self.count_list[(self.down<=candidate[i]) & (self.down>start)])                                                    
                bad_down = sum(self.bad_list[(self.down>candidate[i]) & (self.down<=end)])
                count_down = sum(self.count_list[(self.down>candidate[i]) & (self.down<=end)])   
                
            if count_down == 0:
                rate_down = 0
            else:
                rate_down = bad_down/count_down
            if count_up == 0:
                rate_up = 0
            else:
                rate_up = bad_up/count_up    
                
            rate = rate_down - rate_up
            
            if trend=='up':
                if rate >=0:
                    if not result_rate:
                        result_rate = rate
                        result_cut = candidate[i] 
                    if rate > result_rate and candidate[i] not in self.cut_range:
                        result_rate = rate
                        result_cut = candidate[i] 
            elif trend=='down':
                if rate <=0:
                    if not result_rate:
                        result_rate = rate
                        result_cut = candidate[i] 
                    if rate < result_rate and candidate[i] not in self.cut_range:
                        result_rate = rate
                        result_cut = candidate[i]                 
        #print(result_cut, result_rate)                                               
        return result_cut, result_rate



# -*- coding: utf-8 -*-
from .trendSplit import *
import numpy as np
import copy
import math
from scipy import stats

class backwardSplit(trendSplit):
    def __init__(self, x, y, bad=1,missing=None, force=False):
        trendSplit.__init__(self, x, y, bad, missing, force)

    def fit(self, init_split=0, num_split=0, minv=0, sby='iv'):
        '''
        :param init_split: 备选初始切割点数
        :param num_split: 最大切割点数,不包含最大最小值
        :param minv: 最小分裂所需数值，woe/iv
        :param sby: 'iv','chi'
        :param min_sample: 每个分箱最小样本数
        :return: numpy array -- 切割点数组
        '''
        self.set_init()

        if init_split == 0 or len(self.x) <= init_split:
            self.everysplit()
        else:
            self.equalSize(init_split)

        candidate = []
        for r in self.range_dict:
            candidate.append(r[0])
            candidate.append(r[1])
        self.candidate = sorted(list(set(candidate)))

        param = {'minv': minv, 'sby': sby}
        cut = self.find_cut(**param)

        if cut:
            self.candidate.remove(cut)
            while True:
                cut = self.find_cut(**param)

                if cut:
                    self.candidate.remove(cut)
                else:
                    break

                if num_split:
                    #print(len(set(self.candidate)),num_split, self.candidate)
                    #print()
                    if len(set(self.candidate)) <= num_split:
                        break

            self.bins = np.array(sorted(list(set(self.candidate))))
        else:
            self.bins = None


    def find_cut(self,minv=0, num_split=0, sby='iv'):
        '''
        :param minv: 最小分裂所需数值，woe/iv
        :param sby: 'iv','chi'
        :param num_split: 最大切割点数,不包含最大最小值
        :return:
        '''

        cut = None
        if sby == 'chi':
            minv = 10**7
        if len(self.candidate)>num_split:
            for i in range(1, len(self.candidate)-1):
                candidate_list = [c for c in self.candidate if c!=self.candidate[i]]
                down_idx = candidate_list.index(self.candidate[i+1])
                iv_range = tuple(candidate_list)
                near_range_before = (self.candidate[i-1], self.candidate[i], self.candidate[i+1])

                if sby=='iv':
                    iv = self.cal_iv_by_range(iv_range)
                    if iv > minv:
                        minv = iv
                        cut = self.candidate[i]
                elif sby=='chi':
                    chi_v = self.cal_chisquare_by_range(near_range_before)
                    if chi_v < minv:
                        minv = chi_v
                        cut = self.candidate[i]
        return cut

    def fit_by_spearman(self, init_split=0, min_v=10):
        '''
        :param init_split:
        :param max_v:
        :return:
        '''
        target_dict = None
        self.set_init()
        if init_split == 0 or len(self.x) <= init_split:
            n_split = int(len(self.x)/2)
        else:
            n_split = init_split

        while n_split>=min_v:
            self.equalSize(n_split)
            x_mean = []
            y_mean = []
            for r in self.range_dict:
                if self.range_dict[r] == max(self.range_dict.values()):
                    x_mean.append(np.nanmean(self.x[(self.x <= r[1]) & (self.x >= r[0])]))
                    y_mean.append(np.nanmean(self.value[(self.x <= r[1]) & (self.x >= r[0])]))
                else:
                    x_mean.append(np.nanmean(self.x[(self.x < r[1]) & (self.x >= r[0])]))
                    y_mean.append(np.nanmean(self.value[(self.x < r[1]) & (self.x >= r[0])]))

            #print(stats.spearmanr(x_mean, y_mean))
            if abs(stats.spearmanr(x_mean, y_mean)[0]) > 0.999:
                target_dict = self.range_dict
            n_split -= 1

        candidate = []
        for r in target_dict:
            candidate.append(r[0])
            candidate.append(r[1])
        self.candidate = sorted(list(set(candidate)))
        self.bins = np.array(sorted(list(set(self.candidate))))


# -*- coding: utf-8 -*-
from .simpleMethods import *
import numpy as np
import copy
import math


class trendSplit(simpleMethods):
    def __init__(self, x, y, bad=1,missing=None,force=False):
        simpleMethods.__init__(self, x,  missing=missing,force=force)
        self.y = y
        self.bad=bad
        self.set_init()

    def set_init(self):
        self.cut_range = []
        self.trend = None
        self.value = np.array(self.y)
        if self.missing == None:
            self.value_miss = None
        else:
            self.value_miss = self.value[self.x_org == self.missing]
            self.value = self.value[self.x_org!=self.missing]

        self.allbad = len(self.value[self.value == self.bad])  # 好样本总数
        self.allgood = len(self.value) - self.allbad  # 坏样本总数
        self.candidate = []
        self.woe_cache = {}
        self.iv_cache = {}
        self.chisquare_cache = {}
        self.know_box = {}

    def cal_woe_by_range(self,wrange):
        '''
        根据切点范围(start, mid, end)计算woe
        :param wrange:
        :param trend:
        :return:
        '''
        woe_up = self.cal_woe_by_start_end(wrange[0], wrange[1])
        woe_down = self.cal_woe_by_start_end(wrange[1], wrange[2])

        if self.trend == 'up':
            woe_sub = woe_up - woe_down
        elif self.trend == 'down':
            woe_sub = woe_down - woe_up
        else:
            woe_sub = abs(woe_down - woe_up)
        return woe_sub

    def cal_iv_by_range(self,vrange):
        '''
        根据切点范围(start, mid, end)计算iv
        :param vrange:
        :param bad:
        :return:
        '''
        iv_split = 0
        result = []
        for j in range(len(vrange)-1):
            if (vrange[j], vrange[j+1]) not in self.iv_cache:
                vvalue = self.value[(self.x < vrange[j+1]) & (self.x >= vrange[j])]
                iv_box = self._cal_iv(vvalue)
                self.iv_cache[(vrange[j], vrange[j+1])] = iv_box
            else:
                iv_box = self.iv_cache[(vrange[j], vrange[j+1])]
            result.append(iv_box)
            iv_split += iv_box

        return iv_split

    def cal_woe_by_start_end(self, start, end):
        if (start, end) not in self.woe_cache:
            vvalue = self.value[(self.x < end) & (self.x >= start)]
            woe_box = self._cal_woe(vvalue)
            self.woe_cache[(start, end)] = woe_box
        else:
            woe_box = self.woe_cache[(start, end)]
        return woe_box

    def _cal_woe(self,v):
        '''
        计算woe
        :param v:
        :param bad:
        :return:
        '''
        bad_num = len(v[v == self.bad])
        count_num = len(v)

        if count_num-bad_num==0 or self.allgood==0 or bad_num==0:
            woe = 0
        else:
            woe = math.log((bad_num / (count_num - bad_num)) / (self.allbad / self.allgood))
        return woe

    def _cal_iv(self, v):
        '''
        计算iv
        :param v:
        :param bad:
        :return:
        '''
        bad_num = len(v[v == self.bad])
        count_num = len(v)

        if count_num-bad_num == 0 or self.allgood==0 or bad_num==0:
            iv = 0
        else:
            iv = (bad_num / (count_num - bad_num))*math.log((bad_num / (count_num - bad_num)) / (self.allbad / self.allgood))
        return iv

    def candidateTrend(self,cut_range):
        #print(cut_range)
        trend_up = 0
        trend_down = 0
        result = {}
        if len(cut_range) == 0:
            candidate_list = copy.deepcopy(self.candidate)
        else:
            candidate_list = [self.candidate[0]] + copy.deepcopy(cut_range) + [self.candidate[-1]]
        for i in range(1,len(candidate_list) - 1):
            woe_up = self.cal_woe_by_start_end(candidate_list[i-1], candidate_list[i])
            woe_down = self.cal_woe_by_start_end(candidate_list[i], candidate_list[i+1])

            if woe_up>woe_down:
                trend_up += 1
            elif woe_up<woe_down:
                trend_down += 1
            result[candidate_list[i]] = (woe_up, woe_down)

        if trend_up>trend_down:
            self.trend = 'up'
        elif trend_up<trend_down:
            self.trend = 'down'
        #print(trend_up, trend_down, result)
        return trend_up, trend_down

    def cal_chisquare_by_range(self, chi_range):
        if chi_range not in self.chisquare_cache:
            v_up = self.value[(self.x < chi_range[1]) & (self.x >= chi_range[0])]
            v_down = self.value[(self.x < chi_range[2]) & (self.x >= chi_range[1])]
            all_num = len(v_up)+len(v_down)
            up_bad = len(v_up[v_up==self.bad])
            up_good = len(v_up)-up_bad
            down_bad = len(v_down[v_down==self.bad])
            down_good = len(v_down)-down_bad
            all_g = up_good + down_good
            all_b = up_bad + down_bad

            if len(v_up)==0 or len(v_down)==0:
                chisquare_value = 10**7
            else:
                chisquare_value = (up_bad-len(v_up)*all_b/all_num)/len(v_up)*all_b/all_num + \
                     (up_good-len(v_up)*all_g/all_num)/len(v_up)*all_g/all_num + \
                     (down_good-len(v_down)*all_g/all_num)/len(v_down)*all_g/all_num + \
                     (down_bad-len(v_down)*all_b/all_num)/len(v_down)*all_b/all_num
            self.chisquare_cache[chi_range] = chisquare_value
        else:
            chisquare_value = self.chisquare_cache[chi_range]
        return chisquare_value



# -*- coding: utf-8 -*-
import numpy as np 
import pandas as pd
import math
import copy

class simpleMethods:
    def __init__(self,x, missing=None,force=False):
        self.x_org = x
        self.range_dict = {}
        self.missing=missing
        if self.missing == None:
            self.x = copy.deepcopy(self.x_org)
            self.x_miss = None
        else:
            self.x = self.x_org[self.x_org!=self.missing]
            self.x_miss = self.x_org[self.x_org==self.missing]
        self.force=force
        
    def equalValue(self,size):
        '''
        x 等间距划分分箱  -> (0-0.1,0.1-0.2...)
        :param size:
        :return:
        '''
        self.range_dict = {}

        self.bins = np.linspace(min(self.x), max(self.x), size+1)

        for i in range(len(self.bins)-1):
            self.range_dict[(self.bins[i],self.bins[i+1])] = i

        return self

    def equalHist(self,size):
        '''
        基于np.histogram分箱
        :param size: bin数目
        :return:
        '''
        self.down = {}
        self.hist, self.bins = np.histogram(self.x, bins=size)


        for i in range(len(self.bins)-1):
            start = self.bins[i]
            end = self.bins[i+1]

            self.range_dict[(start, end)] = i       

        return self
    
    def equalSize(self,size):
        '''
        每个分箱样本数平均
        :param size:
        :return:
        '''
        self.range_dict = {}
        # use pandas qcut
        #_, self.bins = pd.qcut(self.x,size,retbins='True',duplicates='drop')
        #self.bins = sorted(list(self.bins))
        # use numpy instead
        breakpoints = np.arange(0, size + 1) / (size) * 100
        self.bins = [np.percentile(self.x, b) for b in breakpoints]

        for i in range(len(self.bins)-1):
            start = self.bins[i]
            end = self.bins[i+1]

            self.range_dict[(start, end)] = i       

        self.bins = np.array(self.bins)
        return self

    def everysplit(self):
        '''
        最细粒度切分
        :return:
        '''
        if len(set(self.x))<=10 and not self.force:
            self.bins = np.array(list(self.x))
        else:
            x_sort = sorted(list(set(self.x)),reverse=False)
            bins = [x_sort[0]]
            for i in range(len(x_sort)-1):
                bins.append((x_sort[i]+x_sort[i+1])/2)
            bins.append(x_sort[-1]+1)
            self.bins = np.array(bins)

        self.range_dict = {}
        for i in range(len(self.bins)-1):
            start = self.bins[i]
            end = self.bins[i+1]

            self.range_dict[(start, end)] = i


# -*- coding: utf-8 -*-
from .trendSplit import *
import numpy as np
import copy
import math


class MAPA(trendSplit):
    def __init__(self, x, y, bad=1):
        trendSplit.__init__(self, x, y, bad)

    def fit(self,trend='up',sby='woe'):
        '''
        :param num_split: 最大切割点数,不包含最大最小值
        :param minv: 最小分裂所需数值，woe/iv
        :param sby: 'woe','iv','woeiv'
        :param min_sample: 每个分箱最小样本数
        :return: numpy array -- 切割点数组
        '''
        self.set_init()
        self.everysplit()
        if trend == 'auto':
            self.candidateTrend()
        else:
            self.trend = trend
        self.test = {}
        candidate = []
        for r in self.range_dict:
            candidate.append(r[0])
            candidate.append(r[1])

        if self.trend == 'up':
            self.candidate = sorted(list(set(candidate)),reverse=False)
        else:
            self.candidate = sorted(list(set(candidate)), reverse=True)

        cut_list, v = self.find_cut(sby=sby)
        self.cut_range = [cut_list[-1]]
        self.cut_range.append(self.candidate[0])
        self.cut_range.append(self.candidate[-1])
        for d in cut_list:
            self.candidate.remove(d)

        while True:
            cut_list, v = self.find_cut(sby=sby)
            if len(cut_list)>0:
                self.cut_range.append(cut_list[-1])
                self.cut_range = sorted(list(set(self.cut_range)))
                for d in cut_list:
                    self.candidate.remove(d)
            else:
                break
        self.bins = np.array(sorted(list(set(self.cut_range))))
        print(self.test)

    def find_cut(self,trend='up',sby='woe'):
        '''
        :param minv: 最小分裂所需数值，woe/iv
        :param sby: 'woe','iv','woeiv'
        :param iv_base: 上一轮的iv值，sby='woe'时不用考虑
        :return:
        '''
        cut_list = []
        cut = None
        minv = 0 # bad rate
        for c in self.candidate:
            if c != self.candidate[0]:
                if trend == 'up':
                    v = self.value[(self.x<c) & (self.x>=self.candidate[0])]
                else:
                    v = self.value[(self.x >=c) & (self.x < self.candidate[0])]

                if len(v)>0:
                    if sby == 'woe':
                        badr = self._cal_woe(v)
                    elif badr == 'bad':
                        badr = len(v[v == self.bad]) / len(v)
                    else:
                        badr = 0
                else:
                    badr = 0
                self.test[c] = badr
                if badr>=minv:
                    minv = badr
                    cut_list.append(c)
                else:
                    break
        return cut_list, minv

    def candidateTrend(self):
        trend_up = 0
        trend_down = 0

        candidate_list = copy.deepcopy(self.candidate)
        for i in range(1, len(candidate_list) - 1):
            v_up = self.value[(self.x < candidate_list[i]) & (self.x >= candidate_list[i - 1])]
            v_down = self.value[(self.x < candidate_list[i + 1]) & (self.x >= candidate_list[i])]
            woe_up = self._cal_woe(v_up)
            woe_down = self._cal_woe(v_down)
            iv_up = self._cal_iv(v_up)
            iv_down = self._cal_iv(v_down)
            if woe_up > woe_down:
                trend_up += 1
            elif woe_up < woe_down:
                trend_down += 1
        if trend_up > trend_down:
            self.trend = 'up'
        else:
            self.trend = 'down'



# -*- coding: utf-8 -*-
from .trendSplit import *
import numpy as np
import copy
import math


class forwardSplit(trendSplit):
    def __init__(self, x, y, bad=1,missing=None, force=False):
        trendSplit.__init__(self, x, y, bad, missing, force)

    def fit(self, init_split=0, num_split=0, minv=0, sby='woe', min_sample=0):
        '''
        :param num_split: 最大切割点数,不包含最大最小值
        :param minv: 最小分裂所需数值，woe/iv
        :param sby: 'woe','iv','woeiv'
        :param min_sample: 每个分箱最小样本数
        :return: numpy array -- 切割点数组
        '''
        self.set_init()

        if init_split == 0 or len(self.x) <= init_split:
            self.everysplit()
        else:
            self.equalSize(init_split)

        candidate = []
        for r in self.range_dict:
            candidate.append(r[0])
            candidate.append(r[1])
        self.candidate = sorted(list(set(candidate)))

        param = {'minv': minv, 'sby': sby, 'min_sample':min_sample}
        cut, iv = self.find_cut(**param)
        param['iv_base'] = iv
        self.cut_range = [cut]
        if cut!=None:
            self.candidate.remove(cut)
            while True:
                cut, iv = self.find_cut(**param)
                param['iv_base'] = iv
                if cut:
                    self.cut_range.append(cut)
                    self.cut_range = sorted(list(set(self.cut_range)))
                    self.candidate.remove(cut)
                else:
                    break

                if num_split:
                    if len(set(self.cut_range)) >= num_split:
                        break

            self.cut_range.append(self.candidate[0])
            self.cut_range.append(self.candidate[-1])
            self.bins = np.array(sorted(list(set(self.cut_range))))
        else:
            self.cut_range = None
            self.bins = None


    def find_cut(self,minv=None, sby='iv',iv_base=0, min_sample=0):
        '''
        :param minv: 最小分裂所需数值，woe/iv
        :param sby: 'woe','iv','woeiv'
        :param iv_base: 上一轮的iv值，sby='woe'时不用考虑
        :return:
        '''
        if not minv:
            minv = 0
        if ((sby== 'woe') or (sby == 'woeiv')) and (not self.trend) and len(self.cut_range)>0:
            self.candidateTrend(self.cut_range)

        iv = 0

        cut = None
        result = {}
        for i in range(1, len(self.candidate) - 1):
            if len(self.cut_range) == 0:
                woe_range = (self.candidate[0]-0.1, self.candidate[i], self.candidate[-1])
                iv_range = (self.candidate[0]-0.1, self.candidate[i], self.candidate[-1])
            else:
                range_list = sorted([self.candidate[0]-0.1, self.candidate[i], self.candidate[-1]]+list(self.cut_range))
                canidx = range_list.index(self.candidate[i])
                #woe_range = (self.candidate[0], self.candidate[i], self.candidate[-1]+1)
                woe_range = (range_list[canidx-1], range_list[canidx], range_list[canidx+1])
                iv_range = tuple(range_list)

            if len(self.value[(self.x < woe_range[1]) & (self.x >= woe_range[0])]) > min_sample and \
                    len(self.value[(self.x < woe_range[2]) & (self.x >= woe_range[1])]) > min_sample:
                if sby == 'woe':
                    woe = self.cal_woe_by_range(woe_range)
                    if woe > minv:
                        minv = woe
                        cut = self.candidate[i]
                elif sby == 'iv':
                    iv = self.cal_iv_by_range(iv_range)
                    result[self.candidate[i]] = iv
                    if iv > minv and iv > iv_base:
                        minv = iv
                        cut = self.candidate[i]
                else:
                    is_trend_tag = False
                    if self.trend in ('up','down'):
                        up_count, down_count = self.candidateTrend(list(iv_range)[1:-1])
                        if (self.trend == 'up' and down_count==0 and up_count>0) or \
                           (self.trend == 'down' and down_count > 0 and up_count == 0):
                            is_trend_tag = True

                    iv = self.cal_iv_by_range(iv_range)
                    #print(is_trend_tag, self.trend not in ('up','down'), self.trend)
                    #print((is_trend_tag or self.trend not in ('up','down')))
                    if (is_trend_tag or self.trend not in ('up','down')) and iv > minv:
                        minv = iv
                        cut = self.candidate[i]

        return cut, iv



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="autoBinning",
    version="0.1.5.1",
    author="Kai Wang",
    author_email="wangkai0112006@163.com",
    description="A small package for feature autoBinning",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kaiwang0112006/autoBinning",
    packages=setuptools.find_packages(),
    install_requires=[  
        'numpy',
        'scipy',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
